freeslot("S_GODZILLA_CROUCH","SPR_GODD")

states[S_GODZILLA_CROUCH] = {
    sprite = SPR_GODD,    -- The crouch sprite
    frame = FF_ANIMATE,   -- Frame animation for crouching
    tics = 1,             -- Time spent in the crouch state
    nextstate = S_PLAY_STND -- Return to standing state after crouch
}

-- Table to track whether a player can crouch
local playerCanCrouch = {}

addHook("MobjDamage", function(target, inflictor, source, damage, damagetype)
    -- Check if the target is a player
    if target.player then
        local player = target.player

        -- Put the player into the pain state if they are crouching
        if target.state == S_GODZILLA_CROUCH then
            target.state = S_PLAY_PAIN
        end

        -- Disable crouching for this player
        playerCanCrouch[player] = false
    end
end)

addHook("ThinkFrame", function()
    for player in players.iterate do
        -- Ensure taunttimer is initialized
        player.taunttimer = player.taunttimer or 0
        player.taunttimer = player.taunttimer + 1

        if player.mo.skin == "godzilla" then
            -- Custom constants
            local CROUCH_HEIGHT = 45 * FRACUNIT  -- Crouching height
            local FIXED_RADIUS = 25 * FRACUNIT  -- Fixed radius

            -- Initialize player's ability to crouch
            if playerCanCrouch[player] == nil then
                playerCanCrouch[player] = true
            end

            -- If the player cannot crouch, restore height and radius to normal
            if not playerCanCrouch[player] then
                player.mo.height = P_GetPlayerHeight(player)  -- Normal height
                player.mo.radius = FIXED_RADIUS  -- Keep radius fixed

                -- Re-enable crouching when the player touches the ground
                if player.mo.z == player.mo.floorz then
                    playerCanCrouch[player] = true
                end

                -- Prevent the player from going into the crouch state
                if player.mo.state == S_GODZILLA_CROUCH then
                    player.mo.state = S_PLAY_STND
                end
            else
                -- Handle crouching when crouch ability is enabled
                if player.cmd.buttons & BT_CUSTOM1 ~= 0 and player.taunttimer >= 1 then
                    -- Enter the crouch state
                    player.mo.state = S_GODZILLA_CROUCH
                    player.mo.height = CROUCH_HEIGHT  -- Set height to crouching height
                    player.mo.radius = FIXED_RADIUS  -- Set radius to fixed value
                    player.taunttimer = 0
                elseif player.mo.state ~= S_GODZILLA_CROUCH then
                    -- Restore normal height and radius when not crouching
                    player.mo.height = P_GetPlayerHeight(player)  -- Normal height
                    player.mo.radius = FIXED_RADIUS  -- Fixed radius
                end
            end
        end
    end
end)