freeslot("MT_HEATBEAM", "SPR_CHRG")

local CHARGE_TIME = TICRATE -- 1 second to charge
local FIRE_INTERVAL = 1 -- Fire every single tic
local HEAT_BEAM_SPEED = 60 -- Speed of the heat beam
local RING_DRAIN_DELAY = 1 * TICRATE -- Start draining rings after 1 second
local RING_DRAIN_INTERVAL = TICRATE / 10 -- Drain rings every 0.1 seconds
local playerChargeState = {} -- Table to track each player's charge state

local function HeatBeam(player)
    if not server then return end

    -- Iterate over all players
    for player in players.iterate do
        -- Check if player and player.mo are valid
        if not (player.mo and player.mo.valid) then
            continue
        end

        -- Ensure the player is using the "godzilla" skin
        if player.mo.skin ~= "godzilla" then continue end

        -- Initialize player's charge state if not present
        if not playerChargeState[player] then
            playerChargeState[player] = {
                chargeTicks = 0,
                fireTicks = 0,
                isFiring = false,
                soundPlayed = false, -- Track whether sound has been played
                ringDrainTicks = 0 -- Timer for ring draining
            }
        end

        local state = playerChargeState[player]

        -- If the player has 0 or 1 rings, stop firing and reset state
        if player.rings <= 1 then
            if state.isFiring then
                -- Reset the state if firing and insufficient rings
                state.chargeTicks = 0
                state.isFiring = false
                state.fireTicks = 0
                state.soundPlayed = false
                state.ringDrainTicks = 0
                player.mo.sprite = SPR_PLAY
                player.mo.frame = A
            end
            continue
        end

        -- When the attack button is held
        if player.cmd.buttons & BT_ATTACK ~= 0 then
            state.chargeTicks = state.chargeTicks + 1

            if state.chargeTicks < CHARGE_TIME then
                -- Charging phase: update sprite
                player.mo.sprite = SPR_CHRG
                player.mo.frame = A
            else
                -- Firing phase
                player.mo.sprite = SPR_CHRG
                player.mo.frame = B

                if not state.isFiring then
                    state.isFiring = true
                    state.fireTicks = 0 -- Reset firing interval
                    -- Play the heat beam sound only once
                    if not state.soundPlayed then
                        S_StartSound(player.mo, sfx_godzbm)
                        state.soundPlayed = true
                    end
                end

                -- Increment fireTicks
                state.fireTicks = state.fireTicks + 1

                -- Spawn a new heat beam every FIRE_INTERVAL tic
                if state.fireTicks >= FIRE_INTERVAL then
                    state.fireTicks = 0 -- Reset fireTicks

                    -- Calculate spawn location 50 units horizontally and 70 units vertically away from player
                    local horizontal_offset = 50 * FRACUNIT -- Horizontal distance
                    local vertical_offset = 70 * FRACUNIT -- Vertical distance (adjusted to 70)
                    local spawnx = player.mo.x + FixedMul(cos(player.mo.angle), horizontal_offset)
                    local spawny = player.mo.y + FixedMul(sin(player.mo.angle), horizontal_offset)
                    local spawnz = player.mo.z + vertical_offset

                    -- Calculate vertical momentum based on aiming
                    local aim_angle = player.aiming -- Aiming is in fine angles

                    local heatBeam = P_SpawnMobj(spawnx, spawny, spawnz, MT_HEATBEAM)

                    -- Check if the heatBeam was successfully spawned
                    if heatBeam and heatBeam.valid then
                        heatBeam.target = player.mo
                        heatBeam.angle = player.mo.angle -- Align angle with player
                        heatBeam.momx = FixedMul(cos(player.mo.angle), 50 * FRACUNIT)
                        heatBeam.momy = FixedMul(sin(player.mo.angle), 50 * FRACUNIT)
                        heatBeam.momz = FixedMul(sin(aim_angle), 50 * FRACUNIT) -- Vertical momentum
                    end
                end

                -- Handle ring draining after 1 second of firing
                if state.chargeTicks >= RING_DRAIN_DELAY then
                    state.ringDrainTicks = state.ringDrainTicks + 1

                    -- Drain a ring every 0.1 seconds
                    if state.ringDrainTicks >= RING_DRAIN_INTERVAL then
                        state.ringDrainTicks = 0 -- Reset the ring drain timer
                        if player.rings > 1 then
                            player.rings = player.rings - 1 -- Reduce rings by 1
                        else
                            -- If only 1 ring left, stop firing and reset charge
                            player.mo.sprite = SPR_PLAY
                            player.mo.frame = A
                            state.chargeTicks = 0
                            state.isFiring = false
                            state.fireTicks = 0
                            state.soundPlayed = false
                            break -- Exit early since firing stops without sufficient rings
                        end
                    end
                end
            end
        else
            -- When the attack button is released, reset the heat beam state
            if state.chargeTicks > 0 then
                -- Reset player state to standing
                player.mo.state = S_PLAY_STND
                state.chargeTicks = 0
                state.isFiring = false
                state.fireTicks = 0
                state.soundPlayed = false -- Reset sound state for next session
                state.ringDrainTicks = 0 -- Reset ring drain timer
            end
        end
    end
end

addHook("ThinkFrame", HeatBeam)