local function random_position_around_player(player)
    -- Random angle in the range 0 to 359 degrees
    local angle = FixedAngle(P_RandomByte() * 360 / 256)
    -- Random radius in the range 0 to 40 units
    local radius = FixedMul(P_RandomByte() * FRACUNIT / 256, 40 * FRACUNIT)
    -- Calculate positions
    local x = player.mo.x + FixedMul(radius, cos(angle))
    local y = player.mo.y + FixedMul(radius, sin(angle))
    local z = player.mo.z + 70 * FRACUNIT
    return x, y, z
end

-- Table to track if the spawn has occurred for each player
local has_spawned = {}

-- Hook into the player thinker to check state and conditions
addHook("ThinkFrame", function()
    for player in players.iterate do
        -- Ensure the player is valid, exists, and has a mobj
        if player.valid and player.mo and player.mo.valid then
            local mo = player.mo

            -- Check if the player is "godzilla"
            if mo.skin == "godzilla" then
                -- If in S_PLAY_PAIN and hasn't spawned yet
                if mo.state == S_PLAY_PAIN and not has_spawned[player] then
                    -- Generate random position around the player
                    local x, y, z = random_position_around_player(player)

                    -- Spawn MT_GODEXP at the calculated position
                    local spawned = P_SpawnMobj(x, y, z, MT_GODEXP)
                    if spawned then
                        -- Play the sound effect
                        S_StartSound(mo, sfx_GODHIT)

                        -- Mark as spawned for this state
                        has_spawned[player] = true
                    end
                elseif mo.state ~= S_PLAY_PAIN then
                    -- Reset the spawn tracker when leaving the S_PLAY_PAIN state
                    has_spawned[player] = false
                end
            end
        end
    end
end)